#include<stdlib.h>
#include<math.h>
#include"jdcnvs.h"

/* jd2date converts julian date to calendar date */
void jd2date(double jd,int *year,int *month,int* day, 
   int *hour,int *minute,int *second)
{
    /* System generated locals */
    double d__1;

    /* Local variables */
    unsigned jdint, tday;
    double secs;
    int sofday;
    double jdf;

/* ********************************************************************** 
*/
/* Parameter_Input: */
/*     double    JD     - Julian Date [Universal Time] */

/* Parameter_Output: */
/*     int year   - Year including century */
/*     int month  - Month */
/*     int day    - Day */
/*     int hour   - Hour */
/*     int second - Second */

/* Purpose: */
/* Convert Julian Date to year, month, day, hour, minute, second */
/* --------------------------------------------------------------------- 
*/
/* **** 2415078.5 = Julian date of 1900.0: */
/* seconds of day */
    d__1 = jd - 2415078.5;
    jdint = (int)(d__1);

/* ***** jdf = Number of seconds of day/86400.0D0 */
    jdf = jd - (int)(jd)/1 + 0.5;
/*     jdf    = JD - DINT(JD) */

    secs = jdf * 86400.;
/* Number of seconds of day [Real*8 */
    sofday = (int) (secs + .5);
/* ***** Convert seconds of day to hour, minute, second: */
/*  "     "     "     "  "  [I*4] */
    *hour=(sofday%86400)/3600;
    *minute=(sofday%3600)/60;
    *second=sofday%60;

    *year = (int) ((jdint * 4. - 1) / 1461.);
    d__1 = (jdint * 4. + 3. - *year * 1461.) / 4.;
    tday = (int)(d__1);
    *month = (int) ((tday * 5. - 3.) / 153.);
    *day = (int) ((tday * 5. + 2. - *month * 153.) / 5.);
    if (*month >= 10) {
	*month += -9;
	++(*year);
    } else {
	*month += 3;
    }
    *year += 1900;
} /* jdut2date_ */

/* jd2mjd converts julian date to modified julian date */
void jd2mjd(double *Jd,double *MJd)
{
  *MJd = *Jd - 2400000.5;
  return;
} /* jd2mjd */

/* mjd2jd converts mod. jul. date to julian date */
void mjd2jd(double *MJd,double *Jd)
{
  *Jd = *MJd + 2400000.5;
} /* mjd2jd */

/* ModJD calculates mod. jul. date from calendar date */
double modjd(int year,int month,int day,int hr,int min,int secs) 
{
  int i=year-1900;
  double l = (month - 14) / 12;
  int r__1 = (int) (((double)(i + l)) * 365.25);
  int amjd = r__1 + (day + 15048 + (month - 2 - l * 12) * 367 / 
    12 + (1202 - i * 12 - month) / 1200);

  double dmjd = (double) amjd + ((double) (hr)) / 24. + 
    ((double) (min)) / 1440. + ((double)(secs))/86400.0;
  return dmjd;
} /* ModJD */

/* mjd2date converts mod. jul. date to calendar date */
void mjd2date(double mjd,int *year,int *month,int* day, 
   int *hour,int *minute,int *second)
{
  double jd;
  
  mjd2jd(&mjd,&jd);
  jd2date(jd,year,month,day,hour,minute,second); 
  return;
}

int YearFromMjd(double mjd)
{
  int y,mo,d,h,mi,s;
  
  mjd2date(mjd,&y,&mo,&d,&h,&mi,&s);
  return y;
}
int DoyFromMjd(double mjd)
{
  int y,mo,d,h,mi,s;
  
  mjd2date(mjd,&y,&mo,&d,&h,&mi,&s);
  return (modjd(y,mo,d,0,0,0)-modjd(y,1,0,0,0,0));
}

int DayFromMjd(double mjd)
{
  int y,mo,d,h,mi,s;
  
  mjd2date(mjd,&y,&mo,&d,&h,&mi,&s);
  return d;
}

int HourFromMjd(double mjd)
{
  int y,mo,d,h,mi,s;
  
  mjd2date(mjd,&y,&mo,&d,&h,&mi,&s);
  return h;
}

int MinuteFromMjd(double mjd)
{
  int y,mo,d,h,mi,s;
  
  mjd2date(mjd,&y,&mo,&d,&h,&mi,&s);
  return mi;
}

int SecondFromMjd(double mjd)
{
  int y,mo,d,h,mi,s;
  
  mjd2date(mjd,&y,&mo,&d,&h,&mi,&s);
  return s;
}

double FracSecondFromMjd(double mjd)
{
  double fracsec;
  fracsec=(mjd*86400.0)-(int)(mjd*86400.0);
  return fracsec;
}

int MonthFromMjd(double mjd)
{
  int y,mo,d,h,mi,s;
  
  mjd2date(mjd,&y,&mo,&d,&h,&mi,&s);
  return mo;
}
char *CharMonthFromMjd(double mjd)
{
  static char *List[] = {"Jan","Feb","Mar","Apr","May","Jun",
                   "Jul","Aug","Sep","Oct","Nov","Dec"};
                   
  return List[MonthFromMjd(mjd)-1];
}

