#include <stdio.h> 
#include <stdlib.h> 
#include <string.h> 
#include <math.h> 
#include "../include/ephi.h"

struct eh ephihdr;
struct ei ephi;

FILE *str_in;

/*
 * Program: cpf_chk
 *
 * Purpose:
 * Reads and tests a CPF file for compliance with the format.
 *
 * Calling sequence:
 *   cpf_chk [-b] cpf_filename
 *     -b gives a brief listing with errors only.
 *
 * History
 *   Nov 10, 2005 - Modify for format version 1.00;
 *                  Add tests for blanks in headers. rlr
 */
main (argc, argv)
     int argc;
     char *argv[];
{
  int i, j;
  int debug= 1;
  int mjd;
  int ftab= 0, nstat, dir;
  int nh1=0, nh2=0, nh3=0, nh4=0, nh5=0, nh9=0, n00=0, n10=0, n11=0, n12=0;
  int n20=0, n21=0, n22=0, n31=0, n32=0, n40=0, n51=0, n52=0, n60=0;
  int n70=0, n99=0;
  int nh1_bl[10], nh1_blerr=0, nh2_bl[25], nh2_blerr=0, nh3_bl[10], nh3_blerr=0;
  int nh4_bl[5], nh4_blerr=0, nh5_bl[5], nh5_blerr=0;
  int n10bad=0, n11bad=0, n12bad=0, n20bad=0, n21bad=0, n22bad=0;
  int n31bad=0, n32bad=0;
  int n40bad=0, n51bad=0, n52bad=0, n60bad=0, n70bad=0;
  int n10low=0, n11low=0, n12low=0, n20low=0, n21low=0, n22low=0;
  int n31low=0, n32low=0;
  int n40low=0, n51low=0, n52low=0, n60low=0, n70low=0;
  int f11=0, f21=0, f31=0, f51=0, n12seq= 0, n22seq= 0, n32seq= 0, n52seq= 0;
  int nesbad= 0;
  int nposrev= 0, nvelrev= 0, naberrev= 0, noffrev= 0;
  int nleapsec= 0;
  int red_flag_only= 0;
  double sod;
  double sec;
  double vec[3], corr;
  char ltab[4];
  char ephstr[256];
  char str[256];
  char tar_namet[11];
  char infilename[256];
  char nvern[5]="1.01";

  if (argc == 3)
    {
      if (strcmp(argv[1],"-b") == 0) red_flag_only= 1;
      strcpy (infilename, argv[2]);
    }
  else
  if (argc == 2)
    {
      strcpy (infilename, argv[1]);
    }
  else
    {
      printf ("Usage: cpf_chk [-b] filename\n");
    }

  if ((str_in = fopen (infilename, "r")) == NULL)
    {
      sprintf (str, "cpf_chk: FATAL! could not open %s\n", infilename);
      perror (str);
      exit (-1);
    }

  /* initialize all the arrays, in case there is nothing to read */
/*printf("ephiinit: init MAXENT= %d\n",MAXENT);*/
  for (i=0; i<MAXENT; i++)
    {
      ephi.c12[i]= 0.e0;
      ephi.c23[i]= 0.e0;
      for (j=0; j<3; j++)
	{
	  ephi.aberoutv[j][i]= 0.e0;
	  ephi.aberbackv[j][i]= 0.e0;
	}
      for (j=0; j<6; j++)
	{
	  ephi.outv[j][i]= 0.e0;
	  ephi.backv[j][i]= 0.e0;
	}
      ephi.leapflag[i]= 0;
      ephi.oscrelcorr[i]= 0.e0;
      for (j=0; j<3; j++)
        {
	  ephi.offsetoutv[j][i]= 0.e0;
	  ephi.offsetbackv[j][i]= 0.e0;
	  ephi.rotangv[j][i]= 0.e0;
	}
      ephi.gast[i]= 0.e0;
      for (j=0; j<2; j++)
	{
	  ephi.eopxy[j][i]= 0.e0;
	}
      ephi.eopdutc[i]= 0.e0;
    }
  
  ephi.nv= -1;
  ephi.nvoff= -1;
  ephi.nvrot= -1;
  ephi.nveop= -1;

/** Read section **/
/** Do not use sscanf, as it can crash program if there is unexpected input. **/
  while(fgets(ephstr,256,str_in) != NULL)
    {
/*printf("ephstr=[%s]\n",ephstr);*/
      if (strncmp(ephstr,"h1",2) == 0 || strncmp(ephstr,"H1",2) == 0)
	{
          if (strncmp(&ephstr[3],"CPF",3) != 0 && strncmp(&ephstr[3],"cpf",3) != 0)
            {
              strncpy(ltab,&ephstr[3],3);
              ftab= 1;
            }
          getifield(ephstr,7,2,&ephihdr.version);
          strncpy(ephihdr.eph_source,&ephstr[11],3);
          getifield(ephstr,15,4,&ephihdr.epyear);
          getifield(ephstr,20,2,&ephihdr.epmon);
          getifield(ephstr,23,2,&ephihdr.epday);
          getifield(ephstr,26,2,&ephihdr.ephour);
          getifield(ephstr,30,4,&ephihdr.eph_seq);
          strncpy(ephihdr.tar_name,&ephstr[35],10);
          strncpy(ephihdr.eph_notes,&ephstr[46],10);

          i= 0;
          if (ephstr[2] != ' ') nh1_bl[i++]= 3;
          if (ephstr[6] != ' ') nh1_bl[i++]= 7;
          if (ephstr[9] != ' ')nh1_bl[i++]= 10;
          if (ephstr[10] != ' ') nh1_bl[i++]= 11;
          if (ephstr[14] != ' ') nh1_bl[i++]= 15;
          if (ephstr[19] != ' ') nh1_bl[i++]= 20;
          if (ephstr[22] != ' ') nh1_bl[i++]= 23;
          if (ephstr[25] != ' ') nh1_bl[i++]= 26;
          if (ephstr[29] != ' ') nh1_bl[i++]= 30;
          nh1_blerr= i;
	  nh1++;
	}
      else 
   if (strncmp(ephstr,"h2",2) == 0 || strncmp(ephstr,"H2",2) == 0)
	{
          getifield(ephstr,3,8,&ephihdr.cospar_id);
          getifield(ephstr,12,4,&ephihdr.sic);
          getifield(ephstr,17,8,&ephihdr.norad);
          getifield(ephstr,26,4,&ephihdr.sttpyear);
          getifield(ephstr,31,2,&ephihdr.sttpmonth);
          getifield(ephstr,34,2,&ephihdr.sttpday);
          getifield(ephstr,37,2,&ephihdr.sttphour);
          getifield(ephstr,40,2,&ephihdr.sttpmin);
          getifield(ephstr,43,2,&ephihdr.sttpsec);
          getifield(ephstr,46,4,&ephihdr.endpyear);
          getifield(ephstr,51,2,&ephihdr.endpmonth);
          getifield(ephstr,54,2,&ephihdr.endpday);
          getifield(ephstr,57,2,&ephihdr.endphour);
          getifield(ephstr,60,2,&ephihdr.endpmin);
          getifield(ephstr,63,2,&ephihdr.endpsec);
          getifield(ephstr,66,5,&ephihdr.ephsep);
          getifield(ephstr,72,1,&ephihdr.compat);
          getifield(ephstr,74,1,&ephihdr.tar_type);
          getifield(ephstr,76,2,&ephihdr.ref_frame);
          getifield(ephstr,80,1,&ephihdr.rot_type);
          getifield(ephstr,82,1,&ephihdr.cofm_app);

          i= 0;
          if (ephstr[2] != ' ') nh2_bl[i++]= 3;
          if (ephstr[11] != ' ') nh2_bl[i++]= 12;
          if (ephstr[16] != ' ') nh2_bl[i++]= 17;
          if (ephstr[25] != ' ') nh2_bl[i++]= 26;
          if (ephstr[30] != ' ') nh2_bl[i++]= 31;
          if (ephstr[33] != ' ') nh2_bl[i++]= 34;
          if (ephstr[36] != ' ') nh2_bl[i++]= 37;
          if (ephstr[39] != ' ') nh2_bl[i++]= 40;
          if (ephstr[42] != ' ') nh2_bl[i++]= 43;
          if (ephstr[45] != ' ') nh2_bl[i++]= 46;
          if (ephstr[50] != ' ') nh2_bl[i++]= 51;
          if (ephstr[53] != ' ') nh2_bl[i++]= 54;
          if (ephstr[56] != ' ') nh2_bl[i++]= 57;
          if (ephstr[59] != ' ') nh2_bl[i++]= 60;
          if (ephstr[62] != ' ') nh2_bl[i++]= 63;
          if (ephstr[65] != ' ') nh2_bl[i++]= 66;
          if (ephstr[71] != ' ') nh2_bl[i++]= 72;
          if (ephstr[73] != ' ') nh2_bl[i++]= 74;
          if (ephstr[75] != ' ') nh2_bl[i++]= 76;
          if (ephstr[78] != ' ') nh2_bl[i++]= 79;
          if (ephstr[80] != ' ') nh2_bl[i++]= 81;
          nh2_blerr= i;
	  nh2++;
	}
      else 
      if (strncmp(ephstr,"h3",2) == 0 || strncmp(ephstr,"H3",2) == 0)
	{
          getifield(ephstr,3,5,&ephihdr.atrk_ro_0);
          getifield(ephstr,9,5,&ephihdr.xtrk_ro_0);
          getifield(ephstr,15,5,&ephihdr.rtrk_ro_0);
          getifield(ephstr,21,5,&ephihdr.atrk_ro_1);
          getifield(ephstr,27,5,&ephihdr.xtrk_ro_1);
          getifield(ephstr,33,5,&ephihdr.rtrk_ro_1);
          getifield(ephstr,39,5,&ephihdr.atrk_ro_2);
          getifield(ephstr,45,5,&ephihdr.xtrk_ro_2);
          getifield(ephstr,51,5,&ephihdr.rtrk_ro_2);

          i= 0;
          if (ephstr[2] != ' ') nh3_bl[i++]= 3;
          if (ephstr[8] != ' ') nh3_bl[i++]= 9;
          if (ephstr[14] != ' ') nh3_bl[i++]= 15;
          if (ephstr[20] != ' ') nh3_bl[i++]= 21;
          if (ephstr[26] != ' ') nh3_bl[i++]= 27;
          if (ephstr[32] != ' ') nh3_bl[i++]= 33;
          if (ephstr[38] != ' ') nh3_bl[i++]= 39;
          if (ephstr[44] != ' ') nh3_bl[i++]= 45;
          if (ephstr[50] != ' ') nh3_bl[i++]= 51;
          nh3_blerr= i;
	  nh3++;
	}
      else 
      if (strncmp(ephstr,"h4",2) == 0 || strncmp(ephstr,"H4",2) == 0)
	{
          getfield(ephstr,3,12,&ephihdr.prf);
          getfield(ephstr,16,10,&ephihdr.xpond_xmit_delay);
          getfield(ephstr,27,11,&ephihdr.xpond_utc_off);
          getfield(ephstr,39,11,&ephihdr.xpond_osc_drift);

          i= 0;
          if (ephstr[2] != ' ') nh4_bl[i++]= 3;
          if (ephstr[15] != ' ') nh4_bl[i++]= 16;
          if (ephstr[26] != ' ') nh4_bl[i++]= 27;
          if (ephstr[38] != ' ') nh4_bl[i++]= 39;
          nh4_blerr= i;
	  nh4++;
	}
      else 
      if (strncmp(ephstr,"h5",2) == 0 || strncmp(ephstr,"H5",2) == 0)
	{
          getfield(ephstr,3,7,&ephihdr.cofm_corr);

          i= 0;
          if (ephstr[2] != ' ') nh5_bl[i++]= 3;
          nh5_blerr= i;
	  nh5++;
	}
      else 
      if (strncmp(ephstr,"h9",2) == 0 || strncmp(ephstr,"H9",2) == 0)
	{
	  nh9++;
	}
      else 
      /* No choice but to use sscanf on free format records */
      if (strncmp(ephstr,"10",2) == 0)
	{
	  if (ephi.nv > MAXENT)
	    {
	      printf("ephemeris read error: too many records (>%d)\n",MAXENT);
	      break;
	    }
	  nstat= sscanf(&ephstr[3],
	    "%d %ld %lf %d %lf %lf %lf",
	    &dir,&mjd,&sod,&ephi.leapflag[ephi.nv],&vec[0],&vec[1],&vec[2]);
          if (ephi.leapflag[ephi.nv] != 0 && ephi.leapflag[ephi.nv] != 1)
		nleapsec++;
	  if (dir == 0)
	    {
	  	(ephi.nv)++;
		ephi.outv[0][ephi.nv]= vec[0];
		ephi.outv[1][ephi.nv]= vec[1];
		ephi.outv[2][ephi.nv]= vec[2];
          	if (nstat == 0) n10bad++;
          	if (nstat > 0 && nstat < 7) n10low++;
		ephi.jdi[ephi.nv]= mjd+ 2400000.0e0;
		ephi.jdf[ephi.nv]= sod/86400.0e0+0.5e0;
		if (ephi.jdf[ephi.nv] >= 1.0e0) 
		  {
		   ephi.jdi[ephi.nv]+= 1.e0;
		   ephi.jdf[ephi.nv]-= 1.e0;
		  }
     		ephi.backv[0][ephi.nv]= -ephi.outv[0][ephi.nv];
		ephi.backv[1][ephi.nv]= -ephi.outv[1][ephi.nv];
		ephi.backv[2][ephi.nv]= -ephi.outv[2][ephi.nv];
	  	n10++;
                if (ephi.nv > 1 && ephihdr.ephsep != 0)
		  if (fabs(((ephi.jdi[ephi.nv]-ephi.jdi[ephi.nv-1])+
			(ephi.jdf[ephi.nv]-ephi.jdf[ephi.nv-1]))*86400.e0
		     - ephihdr.ephsep) > 0.1) nesbad++;
	    }
	  else if (dir == 1)
	    {
	  	(ephi.nv)++;
		ephi.outv[0][ephi.nv]= vec[0];
		ephi.outv[1][ephi.nv]= vec[1];
		ephi.outv[2][ephi.nv]= vec[2];
          	if (nstat == 0) n11bad++;
          	if (nstat > 0 && nstat < 7) n11low++;
		ephi.jdi[ephi.nv]= mjd+ 2400000.0e0;
		ephi.jdf[ephi.nv]= sod/86400.0e0+0.5e0;
		if (ephi.jdf[ephi.nv] >= 1.0e0) 
		  {
		   ephi.jdi[ephi.nv]+= 1.e0;
		   ephi.jdf[ephi.nv]-= 1.e0;
		  }
     		ephi.backv[0][ephi.nv]= -ephi.outv[0][ephi.nv];
		ephi.backv[1][ephi.nv]= -ephi.outv[1][ephi.nv];
		ephi.backv[2][ephi.nv]= -ephi.outv[2][ephi.nv];
	  	n11++;
	  	f11++;
                if (ephi.nv > 1 && ephihdr.ephsep != 0)
		  if (fabs(((ephi.jdi[ephi.nv]-ephi.jdi[ephi.nv-1])+
			(ephi.jdf[ephi.nv]-ephi.jdf[ephi.nv-1]))*86400.e0
		     - ephihdr.ephsep) > 0.1) nesbad++;
	    }
	  else if (dir == 2)
	    {
		ephi.backv[0][ephi.nv]= vec[0];
		ephi.backv[1][ephi.nv]= vec[1];
		ephi.backv[2][ephi.nv]= vec[2];
                if (f11 != 1) n12seq++;
                f11= 0;
          	if (nstat == 0) n12bad++;
          	if (nstat > 0 && nstat < 7) n12low++;

	       /* These are vectors: outbound and back vectors should be of
      		* opposite sign. */
     		if ( ephi.backv[0][ephi.nv]*ephi.outv[0][ephi.nv] > 0.e0 ||
     		     ephi.backv[1][ephi.nv]*ephi.outv[1][ephi.nv] > 0.e0 ||
     		     ephi.backv[2][ephi.nv]*ephi.outv[2][ephi.nv] > 0.e0)
	  	  {
		    nposrev++;
		  }
		n12++;
	    }
	  else
	    {
		n10bad++;
	    }

	}
      else
      if (strncmp(ephstr,"20",2) == 0)
	{
	  nstat= sscanf(&ephstr[3],
	    "%d %lf %lf %lf",
	    &dir,&vec[0],&vec[1],&vec[2]);
	  if (dir == 0)
	    {
	      ephi.outv[3][ephi.nv]= vec[0];
	      ephi.outv[4][ephi.nv]= vec[1];
	      ephi.outv[5][ephi.nv]= vec[2];
              if (nstat == 0) n20bad++;
              if (nstat > 0 && nstat < 4) n20low++;
	      n20++;
	    }
	  if (dir == 1)
	    {
	      ephi.outv[3][ephi.nv]= vec[0];
	      ephi.outv[4][ephi.nv]= vec[1];
	      ephi.outv[5][ephi.nv]= vec[2];
              if (nstat == 0) n21bad++;
              if (nstat > 0 && nstat < 4) n21low++;
	      n21++;
              f21++;
	    }
          else
	  if (dir == 2)
	    {
              if (f21 != 1) n22seq++;
              f21= 0;
	      ephi.backv[3][ephi.nv]= vec[0];
	      ephi.backv[4][ephi.nv]= vec[1];
	      ephi.backv[5][ephi.nv]= vec[2];
	       /* These are vectors: outbound and back vectors should be of
      		* opposite sign. */
     		if ( ephi.backv[3][ephi.nv]*ephi.outv[3][ephi.nv] > 0.e0 ||
     		     ephi.backv[4][ephi.nv]*ephi.outv[4][ephi.nv] > 0.e0 ||
     		     ephi.backv[5][ephi.nv]*ephi.outv[5][ephi.nv] > 0.e0)
	  	  {
		    nvelrev++;
	  	  }
              if (nstat == 0) n22bad++;
              if (nstat > 0 && nstat < 4) n22low++;
	      n22++;
	    }
	  else
	    {
		n20bad++;
	    }
	}
      else
      if (strncmp(ephstr,"30",2) == 0)
	{
	  nstat= sscanf(&ephstr[3],
	    "%d %lf %lf %lf %lf",
	    &dir, &vec[0], &vec[1], &vec[2], &corr);
	  if (dir == 1 || dir == 0)
	    {
		ephi.aberoutv[0][ephi.nv]= vec[0];
		ephi.aberoutv[1][ephi.nv]= vec[1];
	        ephi.aberoutv[2][ephi.nv]= vec[2];
		ephi.c12[ephi.nv]= corr;
          	if (nstat == 0) n31bad++;
          	if (nstat > 0 && nstat < 5) n31low++;
     		/* This is a scalar: Be sure it is positive... */
     		ephi.c12[ephi.nv]= fabs(ephi.c12[ephi.nv]);
     		/* In case there is no return information... */
     		ephi.aberbackv[0][ephi.nv]= -ephi.aberoutv[0][ephi.nv];
     		ephi.aberbackv[1][ephi.nv]= -ephi.aberoutv[1][ephi.nv];
     		ephi.aberbackv[2][ephi.nv]= -ephi.aberoutv[2][ephi.nv];
     		ephi.c23[ephi.nv]= ephi.c12[ephi.nv];	/* scalar */
	  	n31++;
          	f31++;
	    }

          else
      	    if (dir == 2)
	      {
          	if (f31 != 1) n32seq++;
          	f31= 0;
		ephi.aberbackv[0][ephi.nv]= vec[0];
		ephi.aberbackv[1][ephi.nv]= vec[1];
	        ephi.aberbackv[2][ephi.nv]= vec[2];
		ephi.c23[ephi.nv]= corr;
          	if (nstat == 0) n32bad++;
          	if (nstat > 0 && nstat < 4) n32low++;

	       /* These are vectors: outbound and back vectors should be of
      		* opposite sign. */
     		if ( ephi.aberbackv[0][ephi.nv]*ephi.aberoutv[0][ephi.nv] > 0.e0 ||
     		     ephi.aberbackv[1][ephi.nv]*ephi.aberoutv[1][ephi.nv] > 0.e0 ||
     		     ephi.aberbackv[2][ephi.nv]*ephi.aberoutv[2][ephi.nv] > 0.e0)
	  	  {
		    naberrev++;
	  	  }
     		/* This is a scalar: Be sure it is positive... */
     		ephi.c23[ephi.nv]= fabs(ephi.c23[ephi.nv]);
	  	n32++;
	    }
	  else
	    {
		n31bad++;
	    }
	}
      else
      if (strncmp(ephstr,"40",2) == 0)
	{
	  nstat= sscanf(&ephstr[3],
	    "%lf",
	    &ephi.oscrelcorr[ephi.nv]);
          if (nstat == 0) n40bad++;
          if (nstat > 0 && nstat < 1) n40low++;
	  n40++;
	}
      else
      if (strncmp(ephstr,"50",2) == 0)
	{
	  (ephi.nvoff)++;
	  if (ephi.nvoff > MAXENT)
	    {
	      printf("ephemeris read error: too many records\n");
	      break;
	    }
	  nstat= sscanf(&ephstr[3],
	    "%d %ld %lf %d %lf %lf %lf",
	    &dir,&mjd,&sod,&vec[0],&vec[1],&vec[2]);

	  if (dir == 0 ||dir == 1)
	    {
	      ephi.offsetoutv[0][ephi.nvoff]= vec[0];
	      ephi.offsetoutv[1][ephi.nvoff]= vec[1];
	      ephi.offsetoutv[2][ephi.nvoff]= vec[2];
              if (nstat == 0) n51bad++;
              if (nstat > 0 && nstat < 6) n51low++;
	      ephi.offsetjdi[ephi.nvoff]= mjd+ 2400000.0e0;
	      ephi.offsetjdf[ephi.nvoff]= sod/86400.0e0+0.5e0;
	      if (ephi.offsetjdf[ephi.nvoff] >= 1.0e0) 
	        {
	          ephi.offsetjdi[ephi.nvoff]+= 1.e0;
	          ephi.offsetjdf[ephi.nvoff]-= 1.e0;
	        }
       /* In case there is not back vector */
	      ephi.offsetbackv[0][ephi.nv]= -ephi.offsetoutv[0][ephi.nv];
     	      ephi.offsetbackv[1][ephi.nv]= -ephi.offsetoutv[1][ephi.nv];
	      ephi.offsetbackv[2][ephi.nv]= -ephi.offsetoutv[2][ephi.nv];
	      n51++;
              f51++;
	    }
      	  else
          if (dir == 2)
	    {
              if (f51 != 1) n52seq++;
              f51= 0;
	      ephi.offsetbackv[0][ephi.nvoff]= vec[0];
	      ephi.offsetbackv[1][ephi.nvoff]= vec[1];
	      ephi.offsetbackv[2][ephi.nvoff]= vec[2];
              if (nstat == 0) n52bad++;
              if (nstat > 0 && nstat < 6) n52low++;

	       /* These are vectors: outbound and back vectors should be of
      		* opposite sign. */
	      if (ephi.offsetbackv[0][ephi.nvoff]* ephi.offsetoutv[0][ephi.nvoff] > 0.e0 ||
	          ephi.offsetbackv[1][ephi.nvoff]* ephi.offsetoutv[1][ephi.nvoff] > 0.e0 ||
	          ephi.offsetbackv[2][ephi.nvoff]* ephi.offsetoutv[2][ephi.nvoff] > 0.e0)
	        {
		  noffrev++;
	        }
	        n52++;
	    }
	  else
	    {
		n51bad++;
	    }
	}
      else
      if (strncmp(ephstr,"60",2) == 0)
	{
	  (ephi.nvrot)++;
	  if (ephi.nvrot > MAXENT)
	    {
	      printf("ephemeris read error: too many records\n");
	      break;
	    }
	  nstat= sscanf(&ephstr[3],
	    "%ld %lf %lf %lf %lf",
	    &mjd,&sod,&ephi.rotangv[0][ephi.nvrot],
	    &ephi.rotangv[1][ephi.nvrot],&ephi.rotangv[2][ephi.nvrot]);
          if (nstat == 0) n60bad++;
          if (nstat > 0 && nstat < 5) n60low++;

	  ephi.rotjdi[ephi.nvrot]= mjd+ 2400000.0e0;
	  ephi.rotjdf[ephi.nvrot]= sod/86400.0e0+0.5e0;
	  if (ephi.rotjdf[ephi.nvrot] >= 1.0e0) 
	    {
	      ephi.rotjdi[ephi.nvrot]+= 1.e0;
	      ephi.rotjdf[ephi.nvrot]-= 1.e0;
	    }
	  n60++;
	}
      else
      if (strncmp(ephstr,"70",2) == 0)
	{
	  (ephi.nveop)++;
	  if (ephi.nveop > MAXENT)
	    {
	      printf("ephemeris read error: too many records\n");
	      break;
	    }
	  nstat= sscanf(&ephstr[3],
	    "%ld %lf %d %lf %lf %lf",
	    &mjd,&sod,&ephi.eopxy[0][ephi.nveop],
	    &ephi.eopxy[1][ephi.nveop],&ephi.eopdutc[ephi.nveop]);
          if (nstat == 0) n70bad++;
          if (nstat > 0 && nstat < 6) n70low++;

	  ephi.eopjdi[ephi.nveop]= mjd+ 2400000.0e0;
	  ephi.eopjdf[ephi.nveop]= sod/86400.0e0+0.5e0;
	  if (ephi.eopjdf[ephi.nveop] >= 1.0e0) 
	    {
	      ephi.eopjdi[ephi.nveop]+= 1.e0;
	      ephi.eopjdf[ephi.nveop]-= 1.e0;
	    }
	  n70++;
	}
      else
      if (strncmp(ephstr,"99",2) == 0)
	{
	  n99++;
	}
      /* Comment */
      else
      if (strncmp(ephstr,"00",2) == 0)
	{
	  n00++;
	}
    }

/* Recap */
  if (!red_flag_only)
    {
      strncpy (tar_namet, ephihdr.tar_name, 10);
      tar_namet[10]= '\0';
      printf ("CPF Check Version: %s\nCPF File name: %s \n\
Target: COSPAR:%8d SIC:%4d NORAD: %8d Name: [%s]\n\
Sequence No.: %5d By: %s\n\
Produced on %4d/%2d/%2d %2d hours\n\
Starting on %4d/%2d/%2d %2d:%2d:%2d Ending on %4d/%2d/%2d %2d:%2d:%2d\n\
with %5d seconds between entries. Target type: %1d, Compatibility: %1d, \n\
Ref frame: %2d Rotation type: %1d Center of Mass applied: %1d\n\
Notes: [%s]\n", nvern,
infilename,ephihdr.cospar_id,ephihdr.sic,ephihdr.norad,tar_namet,
ephihdr.eph_seq, ephihdr.eph_source,
ephihdr.epyear,ephihdr.epmon,ephihdr.epday,ephihdr.ephour,
ephihdr.sttpyear,ephihdr.sttpmonth,ephihdr.sttpday,ephihdr.sttphour,
ephihdr.sttpmin,ephihdr.sttpsec,
ephihdr.endpyear,ephihdr.endpmonth,ephihdr.endpday,ephihdr.endphour,
ephihdr.endpmin,ephihdr.endpsec,ephihdr.ephsep,
ephihdr.tar_type,ephihdr.compat,ephihdr.ref_frame,ephihdr.rot_type,
ephihdr.cofm_app,ephihdr.eph_notes);

      printf ("\nNumber of records -\n");
      if (n10 > 0)
        printf (" 10 (common epoch): %d\n", n10);
      if (n11 > 0)
        printf (" 10 (outbound position): %d\n", n11);
      if (n12 > 0)
        printf (" 10 (inbound position): %d\n", n12);
      if (n21 > 0)
        printf (" 20 (outbound velocity): %d\n", n21);
      if (n22 > 0)
        printf (" 20 (inbound velocity): %d\n", n22);
      if (n31 > 0)
        printf (" 30 (outbound corrections): %d\n", n31);
      if (n32 > 0)
        printf (" 30 (inbound corrections): %d\n", n32);
      if (n40 > 0)
        printf (" 40 (osc relativity corr): %d\n", n40);
      if (n51 > 0)
        printf (" 50 (outbound offset): %d\n", n51);
      if (n52 > 0)
        printf (" 50 (inbound offset): %d\n", n52);
      if (n60 > 0)
        printf (" 60 (rotation angles): %d\n", n60);
      if (n70 > 0)
        printf (" 70 (earth orientation): %d\n", n70);

      printf("\nErrors and Warnings -\n");
    }

  if (red_flag_only)
      printf ("CPF File name: %s \n\n",infilename);
  if (nh1 == 0) printf ("ERROR: No header H1\n");
  if (nh2 == 0) printf ("ERROR: No header H2\n");
  if (!red_flag_only)
    {
      if (nh3 == 0) printf ("WARNING: No accuracy header H3\n");
      if (nh4 == 0) printf ("NOTE: No transponder header H4\n");
    }
  if (nh9 == 0) printf ("ERROR: No header end record H9\n");
  if (nh1 > 1) printf ("Too many H1: %d\n",nh1);
  if (nh2 > 1) printf ("Too many H2: %d\n",nh2);
  if (nh3 > 1) printf ("Too many H3: %d\n",nh3);
  if (nh4 > 1) printf ("Too many H4: %d\n",nh4);
  if (nh5 > 1) printf ("Too many H5: %d\n",nh5);
  if (nh9 > 1) printf ("Too many H9: %d\n",nh9);
  for (i=0; i<nh1_blerr; i++) 
    printf ("H1 column %d is not blank but should be\n",nh1_bl[i]);
  for (i=0; i<nh2_blerr; i++) 
    printf ("H2 column %d is not blank but should be\n",nh2_bl[i]);
  for (i=0; i<nh3_blerr; i++) 
    printf ("H3 column %d is not blank but should be\n",nh3_bl[i]);
  for (i=0; i<nh4_blerr; i++) 
    printf ("H4 column %d is not blank but should be\n",nh4_bl[i]);
  for (i=0; i<nh5_blerr; i++) 
    printf ("H5 column %d is not blank but should be\n",nh5_bl[i]);

  if (ftab) printf("Header record 1 missing 'CPF' literal. Is [%s] instead.\n",ltab);
  if (n12 > 0 && n12 != n11) printf ("Unmatched in- and outbound position records\n");
  if (n22 > 0 && n22 != n21) printf ("Unmatched in- and outbound velocity records\n");
  if (n32 > 0 && n32 != n31) printf ("Unmatched in- and outbound corrections records\n");
  if (n52 > 0 && n52 != n51) printf ("Unmatched in- and outbound offset records\n");
  if (ephihdr.compat < 0 || ephihdr.compat > 1)
    printf ("TIV Compatibility (value = %d) beyond accepted values of 0 and 1\n",ephihdr.compat);
  if (ephihdr.tar_type < 1 || ephihdr.tar_type > 4)
    printf ("Target type (value = %d) beyond accepted values of 1 and 4\n",
	ephihdr.tar_type);
  if ((ephihdr.tar_type == 3 || ephihdr.tar_type == 4) && nh4 == 0)
    printf("Tranponders require transponder header record H4\n"); 
  if (ephihdr.tar_type == 4 && n40 == 0)
    printf("Asynchronous tranponder may require osc relativistic correction (type 40)\n"); 
  if ((ephihdr.tar_type > 1 && ephihdr.tar_type <= 4) && n31 == 0)
    printf("Target probably requires corrections record(s) 31/32\n");
  if (ephihdr.ref_frame < 0 || ephihdr.ref_frame > 2)
    printf ("Reference frame (value = %d) beyond accepted values of 0 and 2\n",ephihdr.ref_frame);
  if ((ephihdr.ref_frame == 1 || ephihdr.ref_frame == 2) && n70 == 0)
    printf("Inertial reference frame require earth orientation (type 70) records\n");
  if (ephihdr.rot_type < 0 || ephihdr.rot_type > 2)
    printf("Bad rotation type: %d\n",ephihdr.rot_type);
  if (n10bad > 0) 
    printf("Bad input on %d records type 10-0 (common epoch)\n",n10bad);
  if (n11bad > 0) 
    printf("Bad input on %d records type 10-1 (outbound position)\n",n11bad);
  if (n12bad > 0) 
    printf("Bad input on %d records type 10-2 (inbound position)\n",n12bad);
  if (n20bad > 0) 
    printf("Bad input on %d records type 20-0 (outbound velocity)\n",n20bad);
  if (n21bad > 0) 
    printf("Bad input on %d records type 20-1 (outbound velocity)\n",n21bad);
  if (n22bad > 0) 
    printf("Bad input on %d records type 20-2 (inbound velocity)\n",n22bad);
  if (n31bad > 0) 
    printf("Bad input on %d records type 30-1 (outbound correction)\n",n31bad);
  if (n32bad > 0) 
    printf("Bad input on %d records type 30-2 (inbound corrections)\n",n32bad);
  if (n40bad > 0) 
    printf("Bad input on %d records type 40 (osc relativity corr)\n",n40bad);
  if (n51bad > 0) 
    printf("Bad input on %d records type 50-1 (outbound offset)\n",n51bad);
  if (n52bad > 0) 
    printf("Bad input on %d records type 50-2 (inbound offset)\n",n52bad);
  if (n60bad > 0) 
    printf("Bad input on %d records type 60 (rotation angles)\n",n60bad);
  if (n70bad > 0) 
    printf("Bad input on %d records type 70 (earth orientation)\n",n70bad);
  if (n10low > 0) 
    printf("Too few fields on %d records type 10-0 (outbound position)\n",n10low);
  if (n11low > 0) 
    printf("Too few fields on %d records type 10-1 (outbound position)\n",n11low);
  if (n12low > 0) 
    printf("Too few fields on %d records type 10-2 (inbound position)\n",n12low);
  if (n20low > 0) 
    printf("Too few fields on %d records type 20-0 (outbound velocity)\n",n20low);
  if (n21low > 0) 
    printf("Too few fields on %d records type 20-1 (outbound velocity)\n",n21low);
  if (n22low > 0) 
    printf("Too few fields on %d records type 20-2 (inbound velocity)\n",n22low);
  if (n31low > 0) 
    printf("Too few fields on %d records type 30-1 (outbound correction)\n",n31low);
  if (n32low > 0) 
    printf("Too few fields on %d records type 30-2 (inbound corrections)\n",n32low);
  if (n40low > 0) 
    printf("Too few fields on %d records type 40 (osc relativity corr)\n",n40low);
  if (n51low > 0) 
    printf("Too few fields on %d records type 50-1 (outbound offset)\n",n51low);
  if (n52low > 0) 
    printf("Too few fields on %d records type 50-2 (inbound offset)\n",n52low);
  if (n60low > 0) 
    printf("Too few fields on %d records type 60 (rotation angles)\n",n60low);
  if (n70low > 0) 
    printf("Too few fields on %d records type 70 (earth orientation)\n",n70low);
  if (n12seq > 0)
    printf ("Position record 11/12 not in sequence in %d cases\n",n12seq);
  if (n22seq > 0)
    printf ("Velocity record 21/22 not in sequence in %d cases\n",n22seq);
  if (n32seq > 0)
    printf ("Corrections record 31/32 not in sequence in %d cases\n",n32seq);
  if (n52seq > 0)
    printf ("Offset record 51/52 not in sequence in %d cases\n",n52seq);
  if (n99 == 0) printf ("ERROR: No body end record 99\n");
  if (nposrev > 0)
    printf ("Out and inbound position had same signs in %d cases\n",nposrev);
  if (nvelrev > 0)
    printf ("Out and inbound velocity had same signs in %d cases\n",nvelrev);
  if (naberrev > 0)
    printf ("Out and inbound aberration had same signs in %d cases\n",naberrev);
  if (noffrev > 0)
    printf ("Out and inbound offsets had same signs in %d cases\n",noffrev);
  if (nesbad > 0)
    printf ("Record separation disagreed with ephsep in %d cases\n",nesbad);
  if (nleapsec > 0)
    printf ("Leap second flag value bad in %d cases\n",nleapsec);


  printf("\n\n");
#ifdef XXX
/*     create a small usable time tag */
/*printf("ephiinit: create ttag\n");*/
  (ephi.nv)++;
  for (i=0; i<ephi.nv; i++)
    {
        ephi.jds[i]=(ephi.jdf[i]-ephi.jdf[0])+
	       (ephi.jdi[i]-ephi.jdi[0]);
	/**printf("i %d jds %f\n", i, ephi.jds[i]);**/
    }
  (ephi.nvoff)++;
  for (i=0; i<ephi.nvoff; i++)
    {
        ephi.offsetjds[i]=(ephi.offsetjdf[i]-ephi.offsetjdf[0])+
	       (ephi.offsetjdi[i]-ephi.offsetjdi[0]);
    }
  (ephi.nvrot)++;
  for (i=0; i<ephi.nvrot; i++)
    {
        ephi.rotjds[i]=(ephi.rotjdf[i]-ephi.rotjdf[0])+
	       (ephi.rotjdi[i]-ephi.rotjdi[0]);
    }
  (ephi.nveop)++;
  for (i=0; i<ephi.nveop; i++)
    {
        ephi.eopjds[i]=(ephi.eopjdf[i]-ephi.eopjdf[0])+
	       (ephi.eopjdi[i]-ephi.eopjdi[0]);
    }
#endif
}
