/* Program cpfpred/predpkg converted to c:
 * 12/06/05 - rlr.
 */

/*
C  Compute predictions from a cpf file.

C  This is test driver to demonstrate the performance of the ilrs cpf
C  prediction sample code. This software is compatible and rigorous 
C  (to a point) for cpfs for all objects including artificial earth 
C  satellites, reflectors on the moon, and itransponders.
C
C  Program usage: cpfpred -g geodname -i ephname [-o outname] [-s stepsec]
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  08/09/17 - 1) Insure roundoff doesn't cause display of -0 seconds;
C             2) Change test on outer while loop to prevent extra 
C                record from being corrected (consistent with older reference
C                test. rlr
C-----------------------------------------------------------------------*/
/*........... INCLUDE FILES .....................................*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "bounce.h"
#include "dirctn.h"
#include "intrng.h"
#include "obsinf.h"
#include "pnt.h"
#include "../include/ephi.h"
struct eh ephihdr;
struct ei ephi;

double twopi = 6.283185307179586e0;
double cdr = 1.74532925199433e-02;
double casr = 4.84813681109536e-06;
double ctsr = 7.27220521664304e-05;

int debug = 0;

/* Reflector 3 - for test of lunar euler angle records with luncenter */
double feature[3] = { 1735476.394000e0, 3.61132590e0, 26.15524990e0 };
double bodftr[6];

FILE *fd_out;

int cephinit ();
void cnvcoor ();
void jdtgr ();
void predict ();
void read_geod ();
void setup_files ();

void
main (argc, argv)
     int argc;
     char *argv[];
{
  double lhatrn[6], azaltrn[6], lharcv[6], azalrcv[6];
  double tgclat, rgclat;
  double lratrn, lrarcv;
  double tjdint, sutct, fjdutc, tjdutc, secs1, ttltim;
  double stepsec = 900e0;
  double vlight = 0.299792458e+06;
  double minelv = 0.e0;
  int d1[5];
  int nrcd;
  int coloc = 0, outopen = 0, backang = 1;
  int risen = 0;

  char ephname[256], geodname[256], outname[256], outname_full[256];
  char sname[5];

  FILE *fd_out;

/*-----------------------------------------------------------------*/
/*  get file names and open files  */
  setup_files (argc, argv, geodname, ephname, outname, stepsec);

/* Read the CPF ephemeris file */
  if (cephinit (ephname) != 0)
    exit (-1);

/* Convert spherical feature coordinates to rectangular */
  cnvcoor (1.e0, feature, bodftr);

/* Get station coordinates */
  read_geod (geodname, sname, &trad, &tlong, &tgdlat, &tgclat,
	     &rrad, &rlong, &rgdlat, &rgclat, &coloc);
  if (!coloc)
    backang = 1;

/* Load observatory coordinates */
  transm[0] = trad;
  transm[1] = tlong;
  transm[2] = tgclat;
  receiv[0] = rrad;
  receiv[1] = rlong;
  receiv[2] = rgclat;

/*  Insure time starts on even second. */
  tjdint = ephi.jdi[0];

/* jdf is already in utc days */
  sutct = ephi.jdf[0] * 86400.e0 + 5 * ephihdr.ephsep - stepsec;
/* sutct=ephi.jdf[0]*86400.e0 + 5*ephihdr.ephsep- 0.0059136e0;! TEST FOR MARS */

  if (debug)
    printf ("jd= %10.1f %20.12f %20.8f\n", ephi.jdi[0], ephi.jdf[0], sutct);
  nrcd = 1;

/*  Time loop */
  do
    {
      sutct += stepsec;
      fjdutc = sutct / 86400.e0;
      tjdutc = tjdint + fjdutc;
      jdtgr (tjdint, fjdutc, &d1[2], &d1[0], &d1[1], &d1[3], &d1[4], &secs1);

/*  For round-off... */
      if (secs1 >= 59.999995e0)
	{
	  jdtgr (tjdint, fjdutc + 1.e-13, &d1[2], &d1[0], &d1[1], &d1[3],
		 &d1[4], &secs1);
	}
//if (secs1 < 0.e0) printf ("secs1: %.14f\n", secs1);
      if (fabs(secs1) < 1.e-11) secs1= 0.e0;	// to prevent "-0.0" sec
      d1[2] += 1900;

      if (debug)
	printf ("==>date: %d %d %d %d %d %10.6f\n",
		d1[0], d1[1], d1[2], d1[3], d1[4], secs1);

/* Compute range and point angles */
      if (debug)
	printf ("ephsep: %d\n", ephihdr.ephsep);
      predict (tjdint, sutct, vlight, backang,
	       &ttltim, lhatrn, azaltrn, &lratrn, lharcv, azalrcv, &lrarcv);

/*  Check for moon risen or set before writing anything */
      if (azaltrn[2] >= minelv || !risen)
	{
	  if (azaltrn[2] >= minelv)
	    {
	      risen = 1;
	      if (!outopen)
		{
		  outopen = 1;
		  /* Open the listing file  */
		  sprintf (outname_full, "%s_y%02d%02d%02dt%02d%02d", outname,
			   d1[2] % 100, d1[0], d1[1], d1[3], d1[4]);
		  if ((fd_out = fopen (outname_full, "w")) == NULL)
		    {
		      printf ("Could not open file %s\n", outname_full);
		      exit (1);
		    }

		  if (debug)
		    printf ("Writing prediction file: %s\n", outname_full);

		  /* write greeting */
		  fprintf (fd_out,
			   "                  ILRS Laser Predictions Sample Software\n\
                  --------------------------------------\n");
		}

/*  Print time, reflector number, point angles, & range to output file */
	      fprintf (fd_out, "\n Record: %3d   Time:%15.6f   %02d/%02d/%4d  %02d:%02d:%10.7f\n\
   SIC   Az        El        LHA       RA        Dec       Range\n",
		       nrcd, tjdutc, d1[0], d1[1], d1[2], d1[3], d1[4], secs1);

	      if (lratrn < 0.e0)
		lratrn += 24.e0;
	      if (lratrn >= 24.e0)
		lratrn -= 24.e0;

	      if (debug)
		printf (" %5d %10.4f %10.4f %10.4f %10.4f %10.4f %18.11f\n",
			ephihdr.sic, azaltrn[1], azaltrn[2], lhatrn[1],
			lratrn, lhatrn[2], ttltim);
	      fprintf (fd_out,
		       " %5d %9.4f %9.4f %9.4f %9.4f %9.4f %17.11f\n",
		       ephihdr.sic, azaltrn[1], azaltrn[2], lhatrn[1], lratrn,
		       lhatrn[2], ttltim);
	      if (backang)
		fprintf (fd_out,
			 " %5d %9.4f %9.4f %9.4f %9.4f %9.4f %17.11f\n",
			 ephihdr.sic, azalrcv[1], azalrcv[2], lharcv[1],
			 lrarcv, lharcv[2], ttltim);

	      jdtgr (tjdint, fjdutc + 1.e-9 + ttltim / 86400.e0, &d1[2],
		     &d1[0], &d1[1], &d1[3], &d1[4], &secs1);
	      d1[2] += 1900;

	      fprintf (fd_out,
		       "\n Receive:%3d   %20.10f   %02d/%02d/%4d  %02d:%02d:%10.7f\n",
		       nrcd, tjdutc + ttltim / 86400.e0,
		       d1[0], d1[1], d1[2], d1[3], d1[4], secs1);
	      fprintf (fd_out, "Range in km: %20.9f\n",
		       ttltim * vlight / 2.e0);

	      nrcd++;

	    }

/*  Increase time by step size  */
	  if (debug)
	    printf ("Step %f %f %f %f\n", tjdutc, ephi.jdi[ephi.nv - 1],
		    ephi.jdf[ephi.nv - 1],
		    ephi.jdi[ephi.nv - 1] + ephi.jdf[ephi.nv - 1] -
		    5 * ephihdr.ephsep / 86400.e0);

	}
      else if (risen == 1 && outopen == 1)
	{
/*  Close this ephemeris and wait for a new one. */
	  fclose (fd_out);
	  outopen = 0;
	  risen = 0;
	}
      if (debug)
        printf ("Ttest: %f %f %f %d %f %f\n", tjdutc, ephi.jdi[ephi.nv - 1],
	      ephi.jdf[ephi.nv - 1], ephihdr.ephsep, stepsec,
	      ephi.jdi[ephi.nv - 1] + ephi.jdf[ephi.nv - 1] -
	      (5 * ephihdr.ephsep + stepsec) / 86400.e0);
    }
  //while (tjdutc <=
  while (tjdutc <
	 ephi.jdi[ephi.nv - 1] + ephi.jdf[ephi.nv - 1] - (5 * ephihdr.ephsep +
							  stepsec) /
	 86400.e0);

}

/*-----------------------------------------------------------------------
C  GED2GEC computes the geocentric latitude from the geodetic. This is an
C  approximation, although a pretty good one.
C
C  Input: gedlat: Geodetic latitude (dp degrees)
C  Output: <ged2gec>: Geocentric latitude (dp degrees)
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C-----------------------------------------------------------------------*/
double
ged2gec (double gdlat)
{
  double inf = 298.257e0;
  double dlat, f, f2, f3, gl;

  f = 1.e0 / inf;
  f2 = f * f;
  f3 = f * f2;
  gl = gdlat * cdr;
  dlat =
    (f + f2 / 2.e0) * sin (2.e0 * gl) -
    ((f2 + f3) / 2.0e0) * sin (4.e0 * gl) + f3 / 3.e0 * sin (6.e0 * gl);
  return ((gl - dlat) / cdr);
/*  printf ("geod %f, geoc %f, dlat %f\n", gdlat,ged2gec,dlat/cdr); */
}

/*-----------------------------------------------------------------------
C  READ_GEOD 
C  Read the station coordinates from a file.
C
C  Input: 
C	geodname: name of station coordinate file.
C	debug: want printed debug output? (logical; from /deb/.
C
C  Output:
C	sname: station name (char)
C	trad:  Laser transmitter radius (height above geocenter).
C	tlong: Laser transmitter longitude
C	tgdlat:Laser transmitter geodetic latitude
C	tgdcat:Laser transmitter geocentric latitude
C	rrad:  Laser receiver radius (height above geocenter).
C	rlong: Laser receiver longitude
C	rgdlat:Laser receiver geodetic latitude
C	rgdcat:Laser receiver geocentric latitude
C	coloc: Are transmitter and receiver the same? (logical)
C	   (All angles about are d.p. degrees.)
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C-----------------------------------------------------------------------*/
void
read_geod (char *geodname, char *sname, double *trad, double *tlong,
	   double *tgdlat, double *tgclat, double *rrad, double *rlong,
	   double *rgdlat, double *rgclat, int *coloc)
{
  double ged2gec ();
  double flat = 298.255e0, ae = 6.378137e6;
  double ecc2, clng, slng, clat, slat, gdtrad, gdrrad;
  double rheight, theight, xsite, ysite, zsite, xn;
  char str[256];
  FILE *geo_str;

  if ((geo_str = fopen (geodname, "r")) == NULL)
    {
      printf ("Error opening geodetic coordinate file %s\n", geodname);
      exit (1);
    }

  fgets (str, 256, geo_str);
  fgets (str, 256, geo_str);
  if (fgets (str, 256, geo_str) == NULL)
    {
      printf ("Error reading geodetic coordinate file %s\n", geodname);
      exit (1);
    }
  strncpy (sname, &str[22], 4);
  sname[4]= '\0';
  sscanf (&str[40], " %lf %lf %lf %lf %lf %lf %lf %lf",
	  tgdlat, tlong, &theight, tgclat, rgdlat, rlong, &rheight, rgclat);
  if (debug)
    printf ("%s %13.7f %13.7f %13.7f %13.7f %13.7f %13.7f %13.7f %13.7f\n",
	  sname, *tgdlat, *tlong, theight, *tgclat, *rgdlat, *rlong, rheight,
	  *rgclat);

  *coloc = 0;
  if ((*tgdlat != *rgdlat || *tgclat != *rgclat ||
       *tlong != *rlong || theight != rheight) ||
      (*rgdlat != 0.e0 && *rgclat != 0.e0 &&
       *rlong != 0.e0 && rheight != 0.e0))
    *coloc = 1;
  if (debug)
    printf ("coloc= %d\n", *coloc);

/*  In case we want backleg angles anyway. */
  if (*coloc)
    {
      *rgdlat = *tgdlat;
      *rgclat = *tgclat;
      *rlong = *tlong;
      rheight = theight;
    }

/* Get the irritating geocentric latitude for time SDE calculations, etc. */
  if (*tgclat == 0.e0 || *tgclat == *tgdlat)
    *tgclat = ged2gec (*tgdlat);
  if (*rgclat == 0.e0 || *rgclat == *rgdlat)
    *rgclat = ged2gec (*rgdlat);

/* Calculate geocentric rectangular coordinates of site */
  ecc2 = 2.e0 / flat - 1.e0 / (flat * flat);
  clng = cos (*tlong * cdr);
  slng = sin (*tlong * cdr);
  clat = cos (*tgdlat * cdr);
  slat = sin (*tgdlat * cdr);
  xn = ae / sqrt (1.e0 - ecc2 * slat * slat);
  gdtrad = xn + theight;
  xsite = gdtrad * clat * clng;
  ysite = gdtrad * clat * slng;
  zsite = (gdtrad - ecc2 * xn) * slat;
  *trad = sqrt (xsite * xsite + ysite * ysite + zsite * zsite);
  clng = cos (*rlong * cdr);
  slng = sin (*rlong * cdr);
  clat = cos (*rgdlat * cdr);
  slat = sin (*rgdlat * cdr);
  xn = ae / sqrt (1.e0 - ecc2 * slat * slat);
  gdrrad = xn + rheight;
  xsite = gdrrad * clat * clng;
  ysite = gdrrad * clat * slng;
  zsite = (gdrrad - ecc2 * xn) * slat;
  *rrad = sqrt (xsite * xsite + ysite * ysite + zsite * zsite);

  if (debug)
    printf ("rad diff= %f\n", *trad - 6374609.83000e0);
  if (debug)
    printf
      ("%17.8f %17.8f %17.8f %17.8f %17.8f %17.8f %17.8f %17.8f %17.8f %17.8f \n",
       *trad, *tlong, *tgclat, *tgdlat, *rrad, *rlong, *rgclat, *rgdlat,
       theight, rheight);
  if (debug)
    printf ("xn %f, th %f, rh %f, tr %f, rr %f\n", xn, theight, rheight, *trad,
	  *rrad);

  fclose (geo_str);
  return;
}

/*-------------------------------------------------------------------------
**
**	setup_files
**
C Grab arguments from command line.
C  Note: The calls to iargc() and getarg() below are sysem dependent
C        and may need to be replaced...
C
C  Input: none
C  Output:
C	geodname: name of file containing station coordinates
C	ephname:  name of input cpf file
C	outname:  name of output prediction file.
C		  Default is "stdout".
C	stepsec:  Number of seconds between predictions.
C		  Default is ephsep/4.
C
C  Program usage: cpfpred -g geodname -i ephname [-o outname] [-s stepsec]
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C-----------------------------------------------------------------------*/
void
setup_files (int argc, char *argv[], char *geodname, char *ephname,
	     char *outname, double stepsec)
{
  int i;

/*  Get file names  */
  for (i = 1; i < argc; i += 2)
    {
      if (argv[i][0] == '-')
	{
	  if (argv[i][1] == 'g')
	    {
	      strcpy (geodname, argv[i + 1]);
	    }
	  else if (argv[i][1] == 'i')
	    {
	      strcpy (ephname, argv[i + 1]);
	    }
	  else if (argv[i][1] == 'o')
	    {
	      strcpy (outname, argv[i + 1]);
	    }
	  else if (argv[i][1] == 's')
	    {
	      stepsec = atof (argv[i + 1]);
	    }
	}
    }

}
