#include <stdio.h>
#include <math.h>
/* =========================================================================
CC
CC NAME       :  DDREH
CC
CC    CALL DDREH(IAXE,WINKEL,DRMAT)
CC
CC PURPOSE    :  COMPUTES A MATRIX OF A PARTICULAR ROTATION IN 3-D
CC               EUCLIDEAN SPACE.
CC
CC PARAMETERS :
CC         IN :  IAXE   : ROTATION AXIS (1,2 OR 3)            I*4
CC               WINKEL : ANGLE                               R*8
CC        OUT :  DRMAT  : RESULTING 3*3 ROTATION MATRIX       R*8(3,3)
CC
CC SR CALLED  :  ---
CC
CC REMARKS    :  ---
CC
CC AUTHOR     :  G.BEUTLER, M.ROTHACHER
CC
CC VERSION    :  3.0
CC
CC CREATED    :  87/10/30 15:22        LAST MODIFIED :  87/10/30 17:28
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1987      UNIVERSITY OF BERNE
CC                    SWITZERLAND
CC
CC MODIFIED:
CC              17-Oct-2005 : Converted to c; R. Ricklefs UT/CSR
C* ======================================================================= */

void
ddreh(int iaxe, double winkel, double drmat[3][3])
{
      double s, c;
      int i, iaxem1;

      iaxem1= iaxe- 1;  /* for 'c' language conversion */
      s= sin(winkel);
      c= cos(winkel);
      drmat[0][0]=c;
      drmat[1][1]=c;
      drmat[2][2]=c;
      drmat[0][1]=-s;
      drmat[1][2]=-s;
      drmat[2][0]=-s;
      drmat[1][0]=s;
      drmat[2][1]=s;
      drmat[0][2]=s;
      for (i=0; i<3; i++) {
        drmat[i][iaxem1]=0.e0;
        drmat[iaxem1][i]=0.e0;
      }
      drmat[iaxem1][iaxem1]= 1.e0;
}
